"""Define package errors."""
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class SprinklError(Exception):
    """Base class for all errors."""

    pass


class InvalidWebhookEvent(Exception):
    """Invalid webhook event."""

    pass


class AuthenticateError(SprinklError):
    """Authentication error."""

    pass


class TokenExpired(SprinklError):
    """Token expired error."""

    pass


class RequestTimeout(SprinklError):
    """Request timeout."""

    pass


class RequestError(SprinklError):
    """Request error."""

    pass


class ControllerAlreadyRunning(SprinklError):
    """Controller already running error."""

    pass
